using System;
using System.Collections;
using Server;
using Server.Items;
using Server.Targeting;
using Server.daat99;

namespace Server.Engines.Harvest
{
	public abstract class HarvestSystem
	{
		private ArrayList m_Definitions;

		public ArrayList Definitions{ get{ return m_Definitions; } }

		public HarvestSystem()
		{
			m_Definitions = new ArrayList();
		}

		public virtual bool CheckTool( Mobile from, Item tool )
		{
			bool wornOut = ( tool == null || tool.Deleted || (tool is IUsesRemaining && ((IUsesRemaining)tool).UsesRemaining <= 0) );

			if ( wornOut )
				from.SendLocalizedMessage( 1044038 ); // You have worn out your tool!

			return !wornOut;
		}

		public virtual bool CheckHarvest( Mobile from, Item tool )
		{
			return CheckTool( from, tool );
		}

		public virtual bool CheckHarvest( Mobile from, Item tool, HarvestDefinition def, object toHarvest )
		{
			return CheckTool( from, tool );
		}

		public virtual bool CheckRange( Mobile from, Item tool, HarvestDefinition def, Map map, Point3D loc, bool timed )
		{
			bool inRange = ( from.Map == map && from.InRange( loc, def.MaxRange ) );

			if ( !inRange )
				def.SendMessageTo( from, timed ? def.TimedOutOfRangeMessage : def.OutOfRangeMessage );

			return inRange;
		}

		public virtual bool CheckResources( Mobile from, Item tool, HarvestDefinition def, Map map, Point3D loc, bool timed )
		{
			HarvestBank bank = def.GetBank( map, loc.X, loc.Y );
			bool available = ( bank != null && bank.Current >= def.ConsumedPerHarvest );

			if ( !available )
				def.SendMessageTo( from, timed ? def.DoubleHarvestMessage : def.NoResourcesMessage );

			return available;
		}

		public virtual void OnBadHarvestTarget( Mobile from, Item tool, object toHarvest )
		{
		}

		public virtual object GetLock( Mobile from, Item tool, HarvestDefinition def, object toHarvest )
		{
			/* Here we prevent multiple harvesting.
			 * 
			 * Some options:
			 *  - 'return tool;' : This will allow the player to harvest more than once concurrently, but only if they use multiple tools. This seems to be as OSI.
			 *  - 'return GetType();' : This will disallow multiple harvesting of the same type. That is, we couldn't mine more than once concurrently, but we could be both mining and lumberjacking.
			 *  - 'return typeof( HarvestSystem );' : This will completely restrict concurrent harvesting.
			 */

			return tool;
		}

		public virtual void OnConcurrentHarvest( Mobile from, Item tool, HarvestDefinition def, object toHarvest )
		{
		}

		public virtual void OnHarvestStarted( Mobile from, Item tool, HarvestDefinition def, object toHarvest )
		{
		}

		public virtual bool BeginHarvesting( Mobile from, Item tool )
		{
			if ( !CheckHarvest( from, tool ) )
				return false;

			from.Target = new HarvestTarget( tool, this );
			return true;
		}

		public virtual void FinishHarvesting( Mobile from, Item tool, HarvestDefinition def, object toHarvest, object locked )
		{
			from.EndAction( locked );

			if ( !CheckHarvest( from, tool ) )
				return;

			int tileID;
			Map map;
			Point3D loc;

			if ( !GetHarvestDetails( from, tool, toHarvest, out tileID, out map, out loc ) )
			{
				OnBadHarvestTarget( from, tool, toHarvest );
				return;
			}
			else if ( !def.Validate( tileID ) )
			{
				OnBadHarvestTarget( from, tool, toHarvest );
				return;
			}
			
			if ( !CheckRange( from, tool, def, map, loc, true ) )
				return;
			else if ( !CheckResources( from, tool, def, map, loc, true ) )
				return;
			else if ( !CheckHarvest( from, tool, def, toHarvest ) )
				return;

			if ( SpecialHarvest( from, tool, def, map, loc ) )
				return;

			HarvestBank bank = def.GetBank( map, loc.X, loc.Y );

			if ( bank == null )
				return;

			HarvestVein vein = bank.Vein;

			if ( vein != null )
				vein = MutateVein( from, tool, def, bank, toHarvest, vein );

			if ( vein == null )
				return;

			HarvestResource primary = vein.PrimaryResource;
			HarvestResource fallback = vein.FallbackResource;
			HarvestResource resource = MutateResource( from, tool, def, map, loc, vein, primary, fallback );
			
			double skillBase = from.Skills[def.Skill].Base;
			double skillValue = from.Skills[def.Skill].Value;

			Type type = null;

			bool b_IsProspected = (bank.Vein).IsProspected;
			
			if ( skillBase >= resource.ReqSkill )
			{
				type = GetResourceType( from, tool, def, map, loc, resource );

				if ( type != null )
					type = MutateType( type, from, tool, def, map, loc, resource );

				string s_Type = "";
				double d_Chances;
				int i_Tokens = 0;

				if (Daat99OWLTR.Ops[1].Setting && def.Skill == SkillName.Mining && (type.IsSubclassOf(typeof(Server.Items.BaseOre)) || type.IsSubclassOf(typeof(Server.Items.BaseGranite))))
				{
					
					if (skillValue == 0)
					{
						type = null;
						from.SendMessage("You can't mine without mining skill.");
						return;
					}
					bool b_IsOre = true;
					if (type.IsSubclassOf(typeof(Server.Items.BaseOre)))
						b_IsOre = true;
					else if (type.IsSubclassOf(typeof(Server.Items.BaseGranite)))
						b_IsOre = false;

					int i_OreSwitch = 0;
					if (skillValue >= 125.0)
						d_Chances = (Utility.RandomMinMax((int)skillValue*10-1249,1250)/10);
					else
						d_Chances = (Utility.RandomMinMax(1,(int)skillValue*10)/10);
					if (b_IsProspected) //make prospector tools raise the ores
						d_Chances = d_Chances + 10.0;
					
					if (Utility.Random(10) == 3) //10% to get iron regardless of skill
						i_OreSwitch = 0;
					else if (d_Chances >= 116.9) //platinum
						i_OreSwitch = 13;
					else if (d_Chances >= 113.3) //electrum
						i_OreSwitch = 12;
					else if (d_Chances >= 109.2) //toxic
						i_OreSwitch = 11;
					else if (d_Chances >= 104.6) //ice
						i_OreSwitch = 10;
					else if (d_Chances >= 98.5) //blaze
						i_OreSwitch = 9;
					else if (d_Chances >= 93.9) //valorite
						i_OreSwitch = 8;
					else if (d_Chances >= 87.8) //verite
						i_OreSwitch = 7;
					else if (d_Chances >= 81.2) //agapite
						i_OreSwitch = 6;
					else if (d_Chances >= 74.1) //gold
						i_OreSwitch = 5;
					else if (d_Chances >= 66.0) //bronze
						i_OreSwitch = 4;
					else if (d_Chances >= 57.9) //copper
						i_OreSwitch = 3;
					else if (d_Chances >= 49.3) //shadow iron
						i_OreSwitch = 2;
					else if (d_Chances >= 40.2) //dull copper
						i_OreSwitch = 1;
					else //iron
						i_OreSwitch = 0;
					
					if (i_OreSwitch == 0) //calculate gains if you mine iron
                        from.CheckSkill( def.Skill, 00.0, 95.0 );
					else //calculate the gains if you mine anything other then iron
						from.CheckSkill( def.Skill, 15.0 + i_OreSwitch * 5, 95.0 + i_OreSwitch * 5 );

					if (b_IsOre)
					{
						switch (i_OreSwitch)
						{
							case 0: default: type = typeof(Server.Items.IronOre); s_Type = "iron ore"; i_Tokens = 1;  break;
							case 1: type = typeof(Server.Items.DullCopperOre); s_Type = "dull copper ore"; i_Tokens = 4; break;
							case 2: type = typeof(Server.Items.ShadowIronOre); s_Type = "shadow iron ore"; i_Tokens = 7; break;
							case 3: type = typeof(Server.Items.CopperOre); s_Type = "copper ore"; i_Tokens = 10; break;
							case 4: type = typeof(Server.Items.BronzeOre); s_Type = "bronze ore"; i_Tokens = 13; break;
							case 5: type = typeof(Server.Items.GoldOre); s_Type = "golden ore"; i_Tokens = 16; break;
							case 6: type = typeof(Server.Items.AgapiteOre); s_Type = "agapite ore"; i_Tokens = 19; break;
							case 7: type = typeof(Server.Items.VeriteOre); s_Type = "verite ore"; i_Tokens = 22; break;
							case 8: type = typeof(Server.Items.ValoriteOre); s_Type = "valorite ore"; i_Tokens = 25; break;
							case 9: type = typeof(Server.Items.BlazeOre); s_Type = "blaze ore"; i_Tokens = 30; break;
							case 10: type = typeof(Server.Items.IceOre); s_Type = "ice ore"; i_Tokens = 35; break;
							case 11: type = typeof(Server.Items.ToxicOre); s_Type = "toxic ore"; i_Tokens = 40; break;
							case 12: type = typeof(Server.Items.ElectrumOre); s_Type = "electrum ore"; i_Tokens = 45; break;
							case 13: type = typeof(Server.Items.PlatinumOre); s_Type = "platinum ore"; i_Tokens = 50; break;
						}
						if (Daat99OWLTR.Ops[5].Setting && Daat99OWLTR.Ops[27].Setting)
						{
							NewDaat99Holder dh = (NewDaat99Holder)daat99.Daat99OWLTR.TempHolders[from];

							if ( type == typeof(Server.Items.PlatinumOre)	&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
							{
								s_Type = "electrum ore";
								type =  typeof(Server.Items.ElectrumOre);
							}
							if ( type == typeof(Server.Items.ElectrumOre)	&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
							{
								s_Type = "toxic ore";
								type =  typeof(Server.Items.ToxicOre);
							}
							if ( type == typeof(Server.Items.ToxicOre)		&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
							{
								s_Type = "ice ore";
								type =  typeof(Server.Items.IceOre);
							}
							if ( type == typeof(Server.Items.IceOre)		&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
							{
								s_Type = "blaze ore";
								type =  typeof(Server.Items.BlazeOre);
							}
							if ( type == typeof(Server.Items.BlazeOre)		&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
							{
								s_Type = "valorite ore";
								type =  typeof(Server.Items.ValoriteOre);
							}
							if ( type == typeof(Server.Items.ValoriteOre)	&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
							{
								s_Type = "verite ore";
								type =  typeof(Server.Items.VeriteOre);
							}
							if ( type == typeof(Server.Items.VeriteOre)		&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
							{
								s_Type = "agapite ore";
								type =  typeof(Server.Items.AgapiteOre);
							}
							if ( type == typeof(Server.Items.AgapiteOre)	&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
							{
								s_Type = "golden ore";
								type =  typeof(Server.Items.GoldOre);
							}
							if ( type == typeof(Server.Items.GoldOre)		&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
							{
								s_Type = "bronze ore";
								type =  typeof(Server.Items.BronzeOre);
							}
							if ( type == typeof(Server.Items.BronzeOre)		&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
							{
								s_Type = "copper ore";
								type =  typeof(Server.Items.CopperOre);
							}
							if ( type == typeof(Server.Items.CopperOre)		&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
							{
								s_Type = "shadow iron ore";
								type =  typeof(Server.Items.ShadowIronOre);
							}
							if ( type == typeof(Server.Items.ShadowIronOre)	&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
							{
								s_Type = "dull copper ore";
								type =  typeof(Server.Items.DullCopperOre);
							}
							if ( type == typeof(Server.Items.DullCopperOre)	&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
							{
								s_Type = "iron ore";
								type =  typeof(Server.Items.IronOre);
							}
						}
					}
					else
					{
						switch (i_OreSwitch)
						{
							case 0: default: type = typeof(Server.Items.Granite); i_Tokens = 2; break;
							case 1: type = typeof(Server.Items.DullCopperGranite); i_Tokens = 8; break;
							case 2: type = typeof(Server.Items.ShadowIronGranite); i_Tokens = 14; break;
							case 3: type = typeof(Server.Items.CopperGranite); i_Tokens = 20; break;
							case 4: type = typeof(Server.Items.BronzeGranite); i_Tokens = 26; break;
							case 5: type = typeof(Server.Items.GoldGranite); i_Tokens = 32; break;
							case 6: type = typeof(Server.Items.AgapiteGranite); i_Tokens = 38; break;
							case 7: type = typeof(Server.Items.VeriteGranite); i_Tokens = 44; break;
							case 8: type = typeof(Server.Items.ValoriteGranite); i_Tokens = 50; break;
							case 9: type = typeof(Server.Items.BlazeGranite); i_Tokens = 60; break;
							case 10: type = typeof(Server.Items.IceGranite); i_Tokens = 70; break;
							case 11: type = typeof(Server.Items.ToxicGranite); i_Tokens = 80; break;
							case 12: type = typeof(Server.Items.ElectrumGranite); i_Tokens = 90; break;
							case 13: type = typeof(Server.Items.PlatinumGranite); i_Tokens = 100; break;
						}
						if (Daat99OWLTR.Ops[5].Setting && Daat99OWLTR.Ops[27].Setting)
						{
							NewDaat99Holder dh = (NewDaat99Holder)daat99.Daat99OWLTR.TempHolders[from];

							if ( type == typeof(Server.Items.PlatinumGranite)	&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
								 type =  typeof(Server.Items.ElectrumGranite);
							if ( type == typeof(Server.Items.ElectrumGranite)	&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
								 type =  typeof(Server.Items.ToxicGranite);
							if ( type == typeof(Server.Items.ToxicGranite)		&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
								 type =  typeof(Server.Items.IceGranite);
							if ( type == typeof(Server.Items.IceGranite)		&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
								 type =  typeof(Server.Items.BlazeGranite);
							if ( type == typeof(Server.Items.BlazeGranite)		&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
								 type =  typeof(Server.Items.ValoriteGranite);
							if ( type == typeof(Server.Items.ValoriteGranite)	&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
								 type =  typeof(Server.Items.VeriteGranite);
							if ( type == typeof(Server.Items.VeriteGranite)		&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
								 type =  typeof(Server.Items.AgapiteGranite);
							if ( type == typeof(Server.Items.AgapiteGranite)	&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
								 type =  typeof(Server.Items.GoldGranite);
							if ( type == typeof(Server.Items.GoldGranite)		&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
								 type =  typeof(Server.Items.BronzeGranite);
							if ( type == typeof(Server.Items.BronzeGranite)		&& dh.Resources.Contains( CraftResources.GetFromType( type )) )
								 type =  typeof(Server.Items.CopperGranite);
							if ( type == typeof(Server.Items.CopperGranite)		&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
								 type =  typeof(Server.Items.ShadowIronGranite);
							if ( type == typeof(Server.Items.ShadowIronGranite)	&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
								 type =  typeof(Server.Items.DullCopperGranite);
							if ( type == typeof(Server.Items.DullCopperGranite)	&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
								type =  typeof(Server.Items.Granite);
						}
					}
				}
				else if (Daat99OWLTR.Ops[2].Setting && def.Skill == SkillName.Lumberjacking)
				{
					if (skillValue == 0)
					{
						type = null;
						from.SendMessage("You can't lumberjack without lumberjacking skill.");
						return;
					}
					int i_LogSwitch = 0;
					
					if (skillValue >= 125.0)
						d_Chances = (Utility.RandomMinMax((int)skillValue*10-1249,1250)/10);
					else
						d_Chances = (Utility.RandomMinMax(1,(int)skillValue*10)/10);

					if (b_IsProspected) //make prospector tools raise the ore
						d_Chances = d_Chances + 10.0;

					if (Utility.Random(10) == 3) //10% to get normal wood regardless of skill
						i_LogSwitch = 0;
					else if (d_Chances >= 116.0) //Petrified
						i_LogSwitch = 11;
					else if (d_Chances >= 111.5) //Redwood
							i_LogSwitch = 10;
					else if (d_Chances >= 106.5) //Purple Heart
							i_LogSwitch = 9;
					else if (d_Chances >= 101.0) //Bamboo
							i_LogSwitch = 8;
					else if (d_Chances >= 95.0) //Ebony
							i_LogSwitch = 7;
					else if (d_Chances >= 88.5) //Zircote
							i_LogSwitch = 6;
					else if (d_Chances >= 81.5) //Oak
							i_LogSwitch = 5;
					else if (d_Chances >= 74.0) //Yew
							i_LogSwitch = 4;
					else if (d_Chances >= 66.0) //Mohogany
							i_LogSwitch = 3;
					else if (d_Chances >= 60.0) //Ash
							i_LogSwitch = 2;
					else if (d_Chances >= 44.0) //Pine
							i_LogSwitch = 1;
					else //regular wood
						i_LogSwitch = 0;
					
					if (i_LogSwitch == 0) //make sure it'll calculate gains if you lumber regular wood
						from.CheckSkill( def.Skill, 00.0, 95.0 );
					else
						from.CheckSkill( def.Skill, 15.0 + i_LogSwitch * 5, 95.0 + i_LogSwitch * 5 );
					
					switch (i_LogSwitch)
					{
						case 0: default: type = typeof(Server.Items.Log); s_Type = "logs"; i_Tokens = 1; break;
						case 1: type = typeof(Server.Items.PineLog); s_Type = "pine logs"; i_Tokens = 4; break;
						case 2: type = typeof(Server.Items.AshLog); s_Type = "ash logs"; i_Tokens = 7; break;
						case 3: type = typeof(Server.Items.MohoganyLog); s_Type = "mohogany logs"; i_Tokens = 10; break;
						case 4: type = typeof(Server.Items.YewLog); s_Type = "yew logs"; i_Tokens = 15; break;
						case 5: type = typeof(Server.Items.OakLog); s_Type = "oak logs"; i_Tokens = 20; break;
						case 6: type = typeof(Server.Items.ZircoteLog); s_Type = "zircote logs"; i_Tokens = 25; break;
						case 7: type = typeof(Server.Items.EbonyLog); s_Type = "ebony logs"; i_Tokens = 30; break;
						case 8: type = typeof(Server.Items.BambooLog); s_Type = "bamboo logs"; i_Tokens = 35; break;
						case 9: type = typeof(Server.Items.PurpleHeartLog); s_Type = "purpleheart logs"; i_Tokens = 40; break;
						case 10: type = typeof(Server.Items.RedwoodLog); s_Type = "redwood logs"; i_Tokens = 45; break;
						case 11: type = typeof(Server.Items.PetrifiedLog); s_Type = "petrified logs"; i_Tokens = 50; break;
					}
					if (Daat99OWLTR.Ops[5].Setting && Daat99OWLTR.Ops[27].Setting)
					{
						NewDaat99Holder dh = (NewDaat99Holder)daat99.Daat99OWLTR.TempHolders[from];

						if ( type == typeof(Server.Items.PetrifiedLog)		&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
						{
							s_Type = "redwood logs";
							type =  typeof(Server.Items.RedwoodLog);
						}
						if ( type == typeof(Server.Items.RedwoodLog)		&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
						{
							s_Type = "purpleheart logs";
							type =  typeof(Server.Items.PurpleHeartLog);
						}
						if ( type == typeof(Server.Items.PurpleHeartLog)	&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
						{
							s_Type = "bamboo logs";
							type =  typeof(Server.Items.BambooLog);
						}
						if ( type == typeof(Server.Items.BambooLog)			&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
						{
							s_Type = "ebony logs";
							type =  typeof(Server.Items.EbonyLog);
						}
						if ( type == typeof(Server.Items.EbonyLog)			&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
						{
							s_Type = "zircote logs";
							type =  typeof(Server.Items.ZircoteLog);
						}
						if ( type == typeof(Server.Items.ZircoteLog)		&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
						{
							s_Type = "oak logs";
							type =  typeof(Server.Items.OakLog);
						}
						if ( type == typeof(Server.Items.OakLog)			&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
						{
							s_Type = "yew logs";
							type =  typeof(Server.Items.YewLog);
						}
						if ( type == typeof(Server.Items.YewLog)			&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
						{
							s_Type = "mohogany logs";
							type =  typeof(Server.Items.MohoganyLog);
						}
						if ( type == typeof(Server.Items.MohoganyLog)		&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
						{
							s_Type = "ash logs";
							type =  typeof(Server.Items.AshLog);
						}
						if ( type == typeof(Server.Items.AshLog)			&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
						{
							s_Type = "pine logs";
							type =  typeof(Server.Items.PineLog);
						}
						if ( type == typeof(Server.Items.PineLog)			&& dh.Resources.Contains( CraftResources.GetFromType( type ) ) )
						{
							s_Type = "logs";
							type =  typeof(Server.Items.Log);
						}
					}
				}
				else if (!( from.CheckSkill( def.Skill, resource.MinSkill, resource.MaxSkill ) ))
					type = null;
				if ( type != null )
				{
					Item item = Construct( type, from );

					if ( item == null )
						type = null;
					else
					{
						if ( item.Stackable )
						{
							if ( map == Map.Felucca && bank.Current >= def.ConsumedPerFeluccaHarvest )
								item.Amount = def.ConsumedPerFeluccaHarvest;
							else
								item.Amount = def.ConsumedPerHarvest;
						}

						bank.Consume( def, item.Amount );
						if (Daat99OWLTR.Ops[1].Setting && def.Skill == SkillName.Mining && (type.IsSubclassOf(typeof(Server.Items.BaseOre)) || type.IsSubclassOf(typeof(Server.Items.BaseGranite))))
						{
							if (type.IsSubclassOf(typeof(Server.Items.BaseOre)))
							{
								if ( Give( from, item, def.PlaceAtFeetIfFull ) )
									from.SendMessage("you dig some {0} and put it in your backpack", s_Type);
								else
								{
									from.SendMessage("Your backpack is full, so the ore you mined is lost.");
									item.Delete();
								}
							}
							else
							{
								if ( Give( from, item, def.PlaceAtFeetIfFull ) )
									from.SendMessage("you carefully extract some workable stone from the ore vein");
								else
								{
									from.SendMessage("Your backpack is full, so the ore you mined is lost.");
									item.Delete();
								}
							}
							if (from.Map == Map.Felucca)
								i_Tokens = (int)(i_Tokens*1.5);
							if (Daat99OWLTR.Ops[15].Setting)
								GiveTokens.RewardTokens(from, i_Tokens);
						}
						else if (Daat99OWLTR.Ops[2].Setting && def.Skill == SkillName.Lumberjacking)
						{
							if ( Give( from, item, def.PlaceAtFeetIfFull ) )
								from.SendMessage("you put some {0} in your backpack", s_Type);
							else
							{
								from.SendMessage("You can't place any wood into your backpack!");
								item.Delete();
							}
							if (from.Map == Map.Felucca)
								i_Tokens = (int)(i_Tokens*1.5);
							if (Daat99OWLTR.Ops[15].Setting)
								GiveTokens.RewardTokens(from, i_Tokens);
						}
						else
						{
							if ( Give( from, item, def.PlaceAtFeetIfFull ) )
							{
								SendSuccessTo( from, item, resource );
							}
							else
							{
								SendPackFullTo( from, item, def, resource );
								item.Delete();
							}
						}
						if ( tool is IUsesRemaining )
						{
							IUsesRemaining toolWithUses = (IUsesRemaining)tool;

							toolWithUses.ShowUsesRemaining = true;

							if ( toolWithUses.UsesRemaining > 0 )
								--toolWithUses.UsesRemaining;

							if ( toolWithUses.UsesRemaining < 1 )
							{
								tool.Delete();
								def.SendMessageTo( from, def.ToolBrokeMessage );
							}
						}
					}
				}
			}

			if ( type == null )
				def.SendMessageTo( from, def.FailMessage );

			if ( this is Lumberjacking || this is Mining )
				OnHarvestFinished( from, tool, def, vein, bank, resource, toHarvest, type );
			else
				OnHarvestFinished( from, tool, def, vein, bank, resource, toHarvest );
		}

		public virtual void OnHarvestFinished( Mobile from, Item tool, HarvestDefinition def, HarvestVein vein, HarvestBank bank, HarvestResource resource, object harvested )
		{
			OnHarvestFinished( from, tool, def, vein, bank, resource, harvested, null );
		}

		public virtual void OnHarvestFinished( Mobile from, Item tool, HarvestDefinition def, HarvestVein vein, HarvestBank bank, HarvestResource resource, object harvested, Type type )
		{
		}

		public virtual bool SpecialHarvest( Mobile from, Item tool, HarvestDefinition def, Map map, Point3D loc )
		{
			return false;
		}

		public virtual Item Construct( Type type, Mobile from )
		{
			try{ return Activator.CreateInstance( type ) as Item; }
			catch{ return null; }
		}

		public virtual HarvestVein MutateVein( Mobile from, Item tool, HarvestDefinition def, HarvestBank bank, object toHarvest, HarvestVein vein )
		{
			return vein;
		}

		public virtual void SendSuccessTo( Mobile from, Item item, HarvestResource resource )
		{
			resource.SendSuccessTo( from );
		}

		public virtual void SendPackFullTo( Mobile from, Item item, HarvestDefinition def, HarvestResource resource )
		{
			def.SendMessageTo( from, def.PackFullMessage );
		}

		public virtual bool Give( Mobile m, Item item, bool placeAtFeet )
		{
			if ( m.PlaceInBackpack( item ) )
				return true;

			if ( !placeAtFeet )
				return false;

			Map map = m.Map;

			if ( map == null )
				return false;

			ArrayList atFeet = new ArrayList();

			foreach ( Item obj in m.GetItemsInRange( 0 ) )
				atFeet.Add( obj );

			for ( int i = 0; i < atFeet.Count; ++i )
			{
				Item check = (Item)atFeet[i];

				if ( check.StackWith( m, item, false ) )
					return true;
			}

			item.MoveToWorld( m.Location, map );
			return true;
		}

		public virtual Type MutateType( Type type, Mobile from, Item tool, HarvestDefinition def, Map map, Point3D loc, HarvestResource resource )
		{
			return from.Region.GetResource( type );
		}

		public virtual Type GetResourceType( Mobile from, Item tool, HarvestDefinition def, Map map, Point3D loc, HarvestResource resource )
		{
			if ( resource.Types.Length > 0 )
				return resource.Types[Utility.Random( resource.Types.Length )];

			return null;
		}

		public virtual HarvestResource MutateResource( Mobile from, Item tool, HarvestDefinition def, Map map, Point3D loc, HarvestVein vein, HarvestResource primary, HarvestResource fallback )
		{
			if ( vein.ChanceToFallback > Utility.RandomDouble() )
				return fallback;

			double skillValue = from.Skills[def.Skill].Value;

			if ( fallback != null && (skillValue < primary.ReqSkill || skillValue < primary.MinSkill) )
				return fallback;

			return primary;
		}

		public virtual bool OnHarvesting( Mobile from, Item tool, HarvestDefinition def, object toHarvest, object locked, bool last )
		{
			if ( !CheckHarvest( from, tool ) )
			{
				from.EndAction( locked );
				return false;
			}

			int tileID;
			Map map;
			Point3D loc;

			if ( !GetHarvestDetails( from, tool, toHarvest, out tileID, out map, out loc ) )
			{
				from.EndAction( locked );
				OnBadHarvestTarget( from, tool, toHarvest );
				return false;
			}
			else if ( !def.Validate( tileID ) )
			{
				from.EndAction( locked );
				OnBadHarvestTarget( from, tool, toHarvest );
				return false;
			}
			else if ( !CheckRange( from, tool, def, map, loc, true ) )
			{
				from.EndAction( locked );
				return false;
			}
			else if ( !CheckResources( from, tool, def, map, loc, true ) )
			{
				from.EndAction( locked );
				return false;
			}
			else if ( !CheckHarvest( from, tool, def, toHarvest ) )
			{
				from.EndAction( locked );
				return false;
			}

			DoHarvestingEffect( from, tool, def, map, loc );

			new HarvestSoundTimer( from, tool, this, def, toHarvest, locked, last ).Start();

			return !last;
		}

		public virtual void DoHarvestingSound( Mobile from, Item tool, HarvestDefinition def, object toHarvest )
		{
			if ( def.EffectSounds.Length > 0 )
				from.PlaySound( Utility.RandomList( def.EffectSounds ) );
		}

		public virtual void DoHarvestingEffect( Mobile from, Item tool, HarvestDefinition def, Map map, Point3D loc )
		{
			from.Direction = from.GetDirectionTo( loc );

			if ( !from.Mounted )
				from.Animate( Utility.RandomList( def.EffectActions ), 5, 1, true, false, 0 );
		}

		public virtual HarvestDefinition GetDefinition( int tileID )
		{
			HarvestDefinition def = null;

			for ( int i = 0; def == null && i < m_Definitions.Count; ++i )
			{
				HarvestDefinition check = (HarvestDefinition)m_Definitions[i];

				if ( check.Validate( tileID ) )
					def = check;
			}

			return def;
		}

		public virtual void StartHarvesting( Mobile from, Item tool, object toHarvest )
		{
			if ( !CheckHarvest( from, tool ) )
				return;

			int tileID;
			Map map;
			Point3D loc;

			if ( !GetHarvestDetails( from, tool, toHarvest, out tileID, out map, out loc ) )
			{
				OnBadHarvestTarget( from, tool, toHarvest );
				return;
			}

			HarvestDefinition def = GetDefinition( tileID );

			if ( def == null )
			{
				OnBadHarvestTarget( from, tool, toHarvest );
				return;
			}

			if ( !CheckRange( from, tool, def, map, loc, false ) )
				return;
			else if ( !CheckResources( from, tool, def, map, loc, false ) )
				return;
			else if ( !CheckHarvest( from, tool, def, toHarvest ) )
				return;

			object toLock = GetLock( from, tool, def, toHarvest );

			if ( !from.BeginAction( toLock ) )
			{
				OnConcurrentHarvest( from, tool, def, toHarvest );
				return;
			}

			new HarvestTimer( from, tool, this, def, toHarvest, toLock ).Start();
			OnHarvestStarted( from, tool, def, toHarvest );
		}

		public virtual bool GetHarvestDetails( Mobile from, Item tool, object toHarvest, out int tileID, out Map map, out Point3D loc )
		{
			if ( toHarvest is Static && !((Static)toHarvest).Movable )
			{
				Static obj = (Static)toHarvest;

				tileID = (obj.ItemID & 0x3FFF) | 0x4000;
				map = obj.Map;
				loc = obj.GetWorldLocation();
			}
			else if ( toHarvest is StaticTarget )
			{
				StaticTarget obj = (StaticTarget)toHarvest;

				tileID = (obj.ItemID & 0x3FFF) | 0x4000;
				map = from.Map;
				loc = obj.Location;
			}
			else if ( toHarvest is LandTarget )
			{
				LandTarget obj = (LandTarget)toHarvest;

				tileID = obj.TileID & 0x3FFF;
				map = from.Map;
				loc = obj.Location;
			}
			else
			{
				tileID = 0;
				map = null;
				loc = Point3D.Zero;
				return false;
			}

			return ( map != null && map != Map.Internal );
		}
	}
}

namespace Server
{
	public interface IChopable
	{
		void OnChop( Mobile from );
	}

	[AttributeUsage( AttributeTargets.Class )]
	public class FurnitureAttribute : Attribute
	{
		public static bool Check( Item item )
		{
			return ( item != null && item.GetType().IsDefined( typeof( FurnitureAttribute ), false ) );
		}

		public FurnitureAttribute()
		{
		}
	}
}